from cffi import FFI

from ..util import get_ivium_dll_path

ffi = FFI()
IVIUM_DLL_PATH = get_ivium_dll_path()
CHAR_ARRAY = "char[]"
DOUBLE_PTR = "double *"
LONG_PTR = "long *"
UTF_ENCODING = "utf-8"


class Base:
    '''Base class for providing shared state and methods'''
    __is_driver_open = False
    __lib = ffi.dlopen(IVIUM_DLL_PATH)

    @staticmethod
    def get_lib():
        return Base.__lib

    @staticmethod
    def is_driver_open() -> bool:
        return Base.__is_driver_open

    @staticmethod
    def set_driver_open(status: bool):
        Base.__is_driver_open = status



from .base import DOUBLE_PTR, LONG_PTR, Base, ffi

ffi.cdef("""
    long __stdcall IV_getcellstatus(long *devcellstatus);
    long __stdcall IV_setconnectionmode(long *value);
    long __stdcall IV_setcellon(long *cellon);
    long __stdcall IV_setpotential(double *value);
    long __stdcall IV_setpotentialWE2(double *value);
    long __stdcall IV_setcurrent(double *value);
    long __stdcall IV_getpotential(double *value);
    long __stdcall IV_setcurrentrange(long *value);
    long __stdcall IV_setcurrentrangeWE2(long *value);
    long __stdcall IV_getcurrent(double *value);
    long __stdcall IV_getcurrentWE2(double *value);
    long __stdcall IV_setfilter(long *value);
    long __stdcall IV_setstability(long *value);
    long __stdcall IV_setbistatmode(long *value);
    long __stdcall IV_setdac(long *channr, double *value);
    long __stdcall IV_getadc(long *channr, double *value);
    long __stdcall IV_setmuxchannel(long *value);
    long __stdcall IV_setdigout(long *value);
    long __stdcall IV_getdigin(long *value);
    long __stdcall IV_setfrequency(double *value);
    long __stdcall IV_setamplitude(double *value);
    long __stdcall IV_getcurrenttrace(long* npoints, double *rate, double *values);
    long __stdcall IV_getcurrentWE2trace(long* npoints, double *rate, double *values);
    long __stdcall IV_getpotentialtrace(long* npoints, double *rate, double *values);
""")


class DirectModeFunctions(Base):

    @staticmethod
    def IV_getcellstatus() -> tuple[int, int]:
        '''Returns cell status labels
            ["I_ovl", "Anin1_ovl","E_ovl", "CellOff_button pressed", "Cell on"]'''
        cell_status_ptr = ffi.new(LONG_PTR)
        result_code = Base.get_lib().IV_getcellstatus(cell_status_ptr)
        return result_code, cell_status_ptr[0]

    @staticmethod
    def IV_setconnectionmode(connection_mode_number: int) -> int:
        ''' Select the connection mode for the currently connected device.
            The available modes depend on the connected device.
            These are all the supported connection modes: 0=off; 1=EStat4EL(default), 2=EStat2EL,
            3=EstatDummy1,4=EStatDummy2,5=EstatDummy3,6=EstatDummy4
            7=Istat4EL, 8=Istat2EL, 9=IstatDummy, 10=BiStat4EL, 11=BiStat2EL'''
        connection_mode_number_ptr = ffi.new(LONG_PTR, connection_mode_number)
        result_code = Base.get_lib().IV_setconnectionmode(
            connection_mode_number_ptr)
        return result_code

    @staticmethod
    def IV_setcellon(cell_on_mode_number: int) -> int:
        '''Set cell on off to close cell relais (0=off;1=on)'''
        cell_on_mode_number_ptr = ffi.new(LONG_PTR, cell_on_mode_number)
        result_code = Base.get_lib().IV_setcellon(
            cell_on_mode_number_ptr)
        return result_code

    @staticmethod
    def IV_setpotential(potential_value: float) -> int:
        '''Set cell potential'''
        potential_value_ptr = ffi.new(DOUBLE_PTR, potential_value)
        result_code = Base.get_lib().IV_setpotential(potential_value_ptr)
        return result_code

    @staticmethod
    def IV_setpotentialWE2(potential_we2_value: float) -> int:
        '''Set BiStat offset potential'''
        potential_we2_value_ptr = ffi.new(DOUBLE_PTR, potential_we2_value)
        result_code = Base.get_lib().IV_setpotentialWE2(potential_we2_value_ptr)
        return result_code

    @staticmethod
    def IV_setcurrent(current_value: float) -> int:
        '''Set cell current (galvanostatic mode)'''
        current_value_ptr = ffi.new(DOUBLE_PTR, current_value)
        result_code = Base.get_lib().IV_setpotentialWE2(current_value_ptr)
        return result_code

    @staticmethod
    def IV_getpotential() -> tuple[int, float]:
        '''Returns measured potential'''
        potential_value_ptr = ffi.new(DOUBLE_PTR)
        result_code = Base.get_lib().IV_getpotential(potential_value_ptr)
        return result_code, potential_value_ptr[0]

    @staticmethod
    def IV_setcurrentrange(current_range_number: int) -> int:
        '''Set current range, 0=10A, 1=1A, etc,'''
        current_range_number_ptr = ffi.new(LONG_PTR, current_range_number)
        result_code = Base.get_lib().IV_setcurrentrange(current_range_number_ptr)
        return result_code

    @staticmethod
    def IV_setcurrentrangeWE2(current_range_number: int) -> int:
        '''Set current range for BiStat, 0=10mA, 1=1mA, etc,'''
        current_range_number_ptr = ffi.new(LONG_PTR, current_range_number)
        result_code = Base.get_lib().IV_setcurrentrangeWE2(
            current_range_number_ptr)
        return result_code

    @staticmethod
    def IV_getcurrent() -> tuple[int, float]:
        '''Returns measured current'''
        current_value_ptr = ffi.new(DOUBLE_PTR)
        result_code = Base.get_lib().IV_getcurrent(current_value_ptr)
        return result_code, current_value_ptr[0]

    @staticmethod
    def IV_getcurrentWE2() -> tuple[int, float]:
        '''Returns measured current from WE2 (bipotentiostat)'''
        current_value_ptr = ffi.new(DOUBLE_PTR)
        result_code = Base.get_lib().IV_getcurrentWE2(current_value_ptr)
        return result_code, current_value_ptr[0]

    @staticmethod
    def IV_setfilter(filter_number: int) -> int:
        '''Set filter, for int :0=1MHz, 1=100kHz, 2=10kHz, 3=1kHz, 4=10Hz'''
        filter_number_ptr = ffi.new(LONG_PTR, filter_number)
        result_code = Base.get_lib().IV_setfilter(filter_number_ptr)
        return result_code

    @staticmethod
    def IV_setstability(stability_number: int) -> int:
        '''Set stability, for int 0=HighSpeed, 1=Standard, 2=HighStability'''
        stability_number_ptr = ffi.new(LONG_PTR, stability_number)
        result_code = Base.get_lib().IV_setstability(stability_number_ptr)
        return result_code

    @staticmethod
    def IV_setbistatmode(value: int) -> int:
        '''REVISE! --> IV_bistat_mode(int) in documentation
            Select mode for BiStat, for int 0=standard, 1=scanning
            This bistat_mode function also can be used to control the Automatic E-ranging function of the instrument;
            0=AutoEranging off; 1=AutoEranging on'''
        value_ptr = ffi.new(LONG_PTR, value)
        result_code = Base.get_lib().IV_setbistatmode(value_ptr)
        return result_code

    @staticmethod
    def IV_setdac(channel_number: int, value: float) -> int:
        '''Set dac on external port, int=0 for dac1, int=1 for dac2'''
        channel_number_ptr = ffi.new(LONG_PTR, channel_number)
        value_ptr = ffi.new(DOUBLE_PTR, value)
        result_code = Base.get_lib().IV_setdac(channel_number_ptr, value_ptr)
        return result_code

    @staticmethod
    def IV_getadc(channel_number: int) -> tuple[int, float]:
        '''REVISE! Returns measured voltage on external ADC port, int=channelnr. 0-7'''
        channel_number_ptr = ffi.new(LONG_PTR, channel_number)
        measured_voltage_ptr = ffi.new(DOUBLE_PTR)
        result_code = Base.get_lib().IV_getadc(
            channel_number_ptr, measured_voltage_ptr)
        return result_code, measured_voltage_ptr[0]

    @staticmethod
    def IV_setmuxchannel(channel_number=0) -> int:
        '''Set channel of multiplexer, int=channelnr. starting from 0(default)'''
        channel_number_ptr = ffi.new(LONG_PTR, channel_number)
        result_code = Base.get_lib().IV_setmuxchannel(channel_number_ptr)
        return result_code

    @staticmethod
    def IV_setdigout(value: int) -> int:
        '''REVISE! Set digital lines on external port, int is bitmask'''
        value_ptr = ffi.new(LONG_PTR, value)
        result_code = Base.get_lib().IV_setdigout(value_ptr)
        return result_code

    @staticmethod
    def IV_getdigin() -> tuple[int, int]:
        '''REVISE! Returns status of digital inputs from external port, int is bitmask'''
        value_ptr = ffi.new(LONG_PTR)
        result_code = Base.get_lib().IV_getdigin(value_ptr)
        return result_code, value_ptr[0]

    @staticmethod
    def IV_setfrequency(frequency: float) -> int:
        frequency_ptr = ffi.new(DOUBLE_PTR, frequency)
        result_code: int = Base.get_lib().IV_setfrequency(frequency_ptr)
        return result_code

    @staticmethod
    def IV_setamplitude(amplitude: float) -> int:
        amplitude_ptr = ffi.new(DOUBLE_PTR, amplitude)
        result_code: int = Base.get_lib().IV_setamplitude(amplitude_ptr)
        return result_code

    @staticmethod
    def IV_getcurrenttrace(points_quantity: int, interval_rate: float) -> tuple[int, float]:
        '''Returns a sequence of measured currents at defined samplingrate
            (npoints, interval, array of double): npoints<=256, interval: 10us to 20ms'''
        points_quantity_ptr = ffi.new(LONG_PTR, points_quantity)
        interval_rate_ptr = ffi.new(DOUBLE_PTR, interval_rate)
        result_ptr = ffi.new(DOUBLE_PTR)
        result_code = Base.get_lib().IV_getcurrenttrace(
            points_quantity_ptr, interval_rate_ptr, result_ptr)
        return result_code, result_ptr[0]

    @staticmethod
    def IV_getcurrentWE2trace(points_quantity: int, interval_rate: float) -> tuple[int, float]:
        '''Returns a sequence of measured WE2 currents at defined samplingrate
            (npoints, interval, array of double): npoints<=256, interval: 10us to 20ms'''
        points_quantity_ptr = ffi.new(LONG_PTR, points_quantity)
        interval_rate_ptr = ffi.new(DOUBLE_PTR, interval_rate)
        result_ptr = ffi.new(DOUBLE_PTR)
        result_code = Base.get_lib().IV_getcurrentWE2trace(
            points_quantity_ptr, interval_rate_ptr, result_ptr)
        return result_code, result_ptr[0]

    @staticmethod
    def IV_getpotentialtrace(points_quantity: int, interval_rate: float) -> tuple[int, float]:
        '''Returns a sequence of measured potentials at defined samplingrate
            (npoints, interval, array of double): npoints<=256, interval: 10us to 20ms'''
        points_quantity_ptr = ffi.new(LONG_PTR, points_quantity)
        interval_rate_ptr = ffi.new(DOUBLE_PTR, interval_rate)
        result_ptr = ffi.new(DOUBLE_PTR)
        result_code = Base.get_lib().IV_getpotentialtrace(
            points_quantity_ptr, interval_rate_ptr, result_ptr)
        return result_code, result_ptr[0]



class IviumSoftNotRunningError(Exception):
    '''Raised when the IviumSoft software is not running'''

    def __init__(self, message="IviumSoft is not currently running. Please, start it."):
        self.message = message
        super().__init__(self.message)



from .base import CHAR_ARRAY, LONG_PTR, UTF_ENCODING, Base, ffi

ffi.cdef("""
    long __stdcall IV_open();
    long __stdcall IV_close();
    long __stdcall IV_MaxDevices();
    long __stdcall IV_selectdevice(long *devnr);
    long __stdcall IV_getdevicestatus();
    long __stdcall IV_readSN(char *sntext);
    long __stdcall IV_connect(long* devconnect);
    long __stdcall IV_VersionHost(long *version);
    long __stdcall IV_VersionDll();
    long __stdcall IV_VersionCheck();
    long __stdcall IV_HostHandle();
    long __stdcall IV_VersionDllFile();
    long __stdcall IV_VersionDllFileStr();
    long __stdcall IV_SelectChannel(long *channel);
""")


class GenericFunctions(Base):
    @staticmethod
    def IV_open() -> int:
        '''Open the driver to manipulate the Ivium software'''
        Base.set_driver_open(True)
        return Base.get_lib().IV_open()

    @staticmethod
    def IV_close() -> int:
        '''Closes the iviumSoft driver'''
        Base.set_driver_open(False)
        return Base.get_lib().IV_close()

    @staticmethod
    def IV_MaxDevices() -> int:
        '''Returns the maximum number of devices that can be managed by IviumSoft'''
        return Base.get_lib().IV_MaxDevices()

    @staticmethod
    def IV_selectdevice(iviumsoft_instance_number: int = 1) -> tuple[int, int]:
        '''It allows to select one instance of the currently running IviumSoft instances'''
        instance_number_ptr = ffi.new(LONG_PTR, iviumsoft_instance_number)
        result_code = Base.get_lib().IV_selectdevice(instance_number_ptr)
        return result_code, instance_number_ptr[0]

    @staticmethod
    def IV_getdevicestatus() -> int:
        '''It returns -1 (no IviumSoft), 0 (not connected), 1 (available_idle), 2 (available_busy),
            3 (no device available)'''
        return Base.get_lib().IV_getdevicestatus()

    @staticmethod
    def IV_readSN() -> tuple[int, str]:
        '''Returns the serial number of the currently selected device'''
        device_serial_number_ptr = ffi.new(CHAR_ARRAY, 16)
        result_code = Base.get_lib().IV_readSN(device_serial_number_ptr)
        return result_code, ffi.string(device_serial_number_ptr).decode(UTF_ENCODING)

    @staticmethod
    def IV_connect(connection_status: int) -> tuple[int, int]:
        '''It connects the currently selected device'''
        connection_status_ptr = ffi.new(LONG_PTR, connection_status)
        result_code = Base.get_lib().IV_connect(connection_status_ptr)
        return result_code, connection_status_ptr[0]

    @staticmethod
    def IV_VersionHost(version_host: int) -> tuple[int, int]:
        '''REVISE!!! Returns the version Host'''
        version_host_ptr = ffi.new(LONG_PTR, version_host)
        result_code = Base.get_lib().IV_VersionHost(version_host_ptr)
        return result_code, version_host_ptr[0]

    @staticmethod
    def IV_VersionDll() -> int:
        '''Returns the version of the IviumSoft dll'''
        return Base.get_lib().IV_VersionDll()

    @staticmethod
    def IV_VersionCheck() -> int:
        '''It returns 1 if the selected instance of IviumSoft is running'''
        return Base.get_lib().IV_VersionCheck()

    @staticmethod
    def IV_HostHandle() -> int:
        '''REVISE!!! Returns Host Handle'''
        return Base.get_lib().IV_HostHandle()

    @staticmethod
    def IV_VersionDllFile() -> int:
        '''REVISE!!! Returns DLL file version'''
        return Base.get_lib().IV_VersionDllFile()

    @staticmethod
    def IV_VersionDllFileStr() -> str:
        '''REVISE!!! Returns DLL file version str'''
        return Base.get_lib().IV_VersionDllFileStr()

    @staticmethod
    def IV_SelectChannel(channel_number: int) -> int:
        '''Sending the integer value communicates with Multichannel control:
            if not yet active, the [int] number of tabs is automatically opened and the [int] tab becomes active;
            if Ivium-n-Soft is active already, the [int] tab becomes active. 
            Now the channel/instrument that is connected to this tab can be controlled. 
            If no instrument is connected, the next available instrument in the list can be connected (IV_connect) and controlled.'''
        channel_number_ptr = ffi.new(LONG_PTR, channel_number)
        result_code = Base.get_lib().IV_SelectChannel(channel_number_ptr)
        return result_code



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['pyvium', 'pyvium.core', 'pyvium.errors', 'pyvium.pyvium', 'pyvium.util']

package_data = \
{'': ['*']}

install_requires = \
['cffi>=1.15.1,<2.0.0']

setup_kwargs = {
    'name': 'pyvium',
    'version': '0.1.6b0',
    'description': 'A tiny Python wrapper around the <Software development driver DLL> for IviumSoft.',
    'long_description': '# PYVIUM\n\nTiny Python wrapper around the "Software development driver DLL" for IviumSoft.\n\n# Important:\n\nThis module uses a dll from the IviumSoft application. You need to have this software installed on a Windows machine. The IviumSoft application can be downloaded from here: https://www.ivium.com/support/#Software%20update\n\nThis version of Pyvium has been tested for IviumSoft release 4.1100.\n\n## Installation\n\nInstall PYVIUM easily with pip:\n\n```\npip install pyvium\n```\n\nOr with poetry:\n\n```\npoetry add pyvium\n```\n\n## Usage Example (Using IviumSoft Core functions)\n\nTo use the same functions available in the "IviumSoft driver DLL" you can import the Core class as follows. All functions return a result code (integer) and a result value if available. For further information you can check the IviumSoft documentation.\n\n```\nfrom pyvium import Core\n\nCore.IV_open()\nCore.IV_getdevicestatus()\nCore.IV_close()\n```\n\n## Usage Example (Using Pyvium methods)\n\nThis is a wrapper around the Core functions that adds a few things:\n\n- Exception management (you can find an example [here](https://github.com/SF-Tec/pyvium/blob/main/docs/error_management.md))\n- New functionalities\n\n```\nfrom pyvium import Pyvium\n\nPyvium.open_driver()\nPyvium.get_device_status()\nPyvium.close_driver()\n\n```\n\n## Supported functions\n\nThe list of currently supported and implemented functions can be found [here](https://github.com/SF-Tec/pyvium/blob/main/docs/method_list.md).\n\n## Links\n\n- [See on GitHub](https://github.com/sf-tec/pyvium)\n- [See on PyPI](https://pypi.org/project/pyvium)\n',
    'author': 'Alejandro Gutiérrez',
    'author_email': 'agutierrez@stec.es',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://github.com/SF-Tec/pyvium',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.11,<4.0',
}


setup(**setup_kwargs)



class DriverNotOpenError(Exception):
    '''Raised when the IviumSoft driver has not been open'''

    def __init__(self, message="The driver is not open. Please, use open_driver() to open it."):
        self.message = message
        super().__init__(self.message)
